/**
 * @file string.h
 * 
 * <p>
 * String class that uses rstm memory allocator is defined in the file.
 * </p>
 * 
 * @author Aleksandar Dragoejvic aleksandar.dragojevic@epfl.ch
 * @date 2007-03-27
 */

#ifndef SB7_STRING_H_
#define SB7_STRING_H_

#include "StlAllocator.h"

namespace sb7 {
	/**
	 * <p>
	 * String type to be used in the benchmark. This type is the same as
	 * standard C++ string class except that it uses stm::StlAllocator for
	 * allocating memory.
	 * </p>
	 * <p>
	 * Objects of this type are NOT shareable in the rstm environment. All
	 * objects of this type should be encapsulated in objects that are if
	 * they should be shared.
	 * </p>
	 * 
	 */
	typedef std::basic_string<char,
			std::char_traits<char>,
			stm::StlAllocator<char> > string;
	
}

#endif /*SB7_STRING_H_*/
