#ifndef SB7_THREAD_H_
#define SB7_THREAD_H_

/**
 * @file thread.h
 * 
 * This file contains definitions of things that are related to thread
 * management and are not defined in appropriate way in rstm library
 * itself.
 * 
 */

#include <pthread.h>

#include "stm_api.h"

#include "../random.h"

namespace sb7 {

	class ThreadData {
		public:
			void* operator new(size_t size) {
				return stm::tx_alloc(size);
			}
 
			void  operator delete(void* p) {
				stm::tx_free(p);
			}

			Random *getRandom() {
				return &random;
			}

		private:
			// random
			Random random;
	};

	/**
	 * Initialize data shared by all threads.
	 */
	void global_thread_init();

	/**
	 * Initialize specific thread.
	 */
	void thread_init();

	/**
	 * Special initialization for init thread.
	 */
	void init_thread_init();

	/**
	 * Clean after the thread.
	 */
	void thread_clean();

	/**
	 * Get thread local thread data.
	 */
	ThreadData *get_thread_data();

	/**
	 * Shortcut for getting thread local random object.
	 */
	Random *get_random();

	/**
	 * Convenience function for delaying thread for time interval
	 * that is specified in milliseconds.
	 */
	bool sleep(int ms);

	/**
	 * Get time in ms since epoch. This function might be architecture specific.
	 */
	long get_time_ms();

	/**
	 * Get time in us since epoch. This function might be architecture specific.
	 */
	long get_time_us();

/**
 * Macro that replaces BEGIN_TRANSACTION from stm.
 */
#define SB7_BEGIN_TRANSACTION	\
	BEGIN_TRANSACTION			\

/**
 * Macro that replaces END_TRANSACTION from stm.
 */
#define SB7_END_TRANSACTION		\
	END_TRANSACTION				\

}

#endif /*SB7_THREAD_H_*/
